@@ -8,7 +8,8 @@
 
 use crate::types::error::GenericTransportError;
 use hyper::client::{Client, HttpConnector};
-use hyper_rustls::HttpsConnector;
+use hyper_rustls::{HttpsConnector, HttpsConnectorBuilder};
+use jsonrpsee_types::CertificateStore;
 use jsonrpsee_utils::http_helpers;
 use thiserror::Error;
 
@@ -27,11 +28,23 @@ pub(crate) struct HttpTransportClient {
 
 impl HttpTransportClient {
 	/// Initializes a new HTTP client.
-	pub(crate) fn new(target: impl AsRef<str>, max_request_body_size: u32) -> Result<Self, Error> {
+	pub(crate) fn new(
+		target: impl AsRef<str>,
+		max_request_body_size: u32,
+		cert_store: CertificateStore,
+	) -> Result<Self, Error> {
 		let target = url::Url::parse(target.as_ref()).map_err(|e| Error::Url(format!("Invalid URL: {}", e)))?;
 		if target.scheme() == "http" || target.scheme() == "https" {
-			let connector = HttpsConnector::with_native_roots();
-			let client = Client::builder().build::<_, hyper::Body>(connector);
+			let connector = match cert_store {
+				CertificateStore::Native => {
+					HttpsConnectorBuilder::new().with_native_roots().https_or_http().enable_http1()
+				}
+				CertificateStore::WebPki => {
+					HttpsConnectorBuilder::new().with_webpki_roots().https_or_http().enable_http1()
+				}
+				_ => return Err(Error::InvalidCertficateStore),
+			};
+			let client = Client::builder().build::<_, hyper::Body>(connector.build());
 			Ok(HttpTransportClient { target, client, max_request_body_size })
 		} else {
 			Err(Error::Url("URL scheme not supported, expects 'http' or 'https'".into()))
@@ -39,7 +52,7 @@ impl HttpTransportClient {
 	}
 
 	async fn inner_send(&self, body: String) -> Result<hyper::Response<hyper::Body>, Error> {
-		log::debug!("send: {}", body);
+		tracing::debug!("send: {}", body);
 
 		if body.len() > self.max_request_body_size as usize {
 			return Err(Error::RequestTooLarge);
@@ -99,6 +112,10 @@ pub(crate) enum Error {
 	/// Malformed request.
 	#[error("Malformed request")]
 	Malformed,
+
+	/// Invalid certificate store.
+	#[error("Invalid certificate store")]
+	InvalidCertficateStore,
 }
 
 impl<T> From<GenericTransportError<T>> for Error
@@ -116,18 +133,18 @@ where
 
 #[cfg(test)]
 mod tests {
-	use super::{Error, HttpTransportClient};
+	use super::{CertificateStore, Error, HttpTransportClient};
 
 	#[test]
 	fn invalid_http_url_rejected() {
-		let err = HttpTransportClient::new("ws://localhost:9933", 80).unwrap_err();
+		let err = HttpTransportClient::new("ws://localhost:9933", 80, CertificateStore::Native).unwrap_err();
 		assert!(matches!(err, Error::Url(_)));
 	}
 
 	#[tokio::test]
 	async fn request_limit_works() {
 		let eighty_bytes_limit = 80;
-		let client = HttpTransportClient::new("http://localhost:9933", 80).unwrap();
+		let client = HttpTransportClient::new("http://localhost:9933", 80, CertificateStore::WebPki).unwrap();
 		assert_eq!(client.max_request_body_size, eighty_bytes_limit);
 
 		let body = "a".repeat(81);
